/* ---------------------------------------------------------- */
/*  basicsystem.cpp                                           */
/* ---------------------------------------------------------- */

/*------------------------------------------------------------- */
/** @file
    @brief		Luna system ̉Bƒۉ
    @author		K.Kunikane (끗ӂ)
    @since		Feb.12.2005
    $Revision: 1.1.1.1 $
*/
/*-----------------------------------------------------
 Copyright (C) 2005 rero2@fumi <rero2@yuumu.org>
 All Rights Reserved.
 ------------------------------------------------------*/

/*-------------------------------*/
/* include                       */
/*-------------------------------*/
#include "basicsystem.h"

/*-------------------------------*/
/* define                        */
/*-------------------------------*/

/* --- TEht@C */
char* sound_name[] = {
  "walk_1.wav",
  "walk_2.wav",
  "walk_3.wav",
  "walk_4.wav",
  "walk_5.wav",
  "walk_6.wav",
  "walk_7.wav",
  "myshot.wav",
  "enemyhit.wav",
  "graze.wav",
  "myshiphit.wav",
  "bonus_hit.wav",
  "start.wav",
};

/* --- ~[WbNt@C */
char* music_name[] = {
  "bonus_ship.wav",
};

/* --- eNX`t@C */
char* texture_name[] = {
  "myship01.png",
  "enemy01.png",
  "bonus01.png",
  "parts01.png",
  "background.png",
  "scorepanel01.png",
  "title01.png",
};

/*-------------------------------*/
/* global value                  */
/*-------------------------------*/

/*-------------------------------*/
/* local routine                 */
/*-------------------------------*/

/*-------------------------------*/
/* implementatiion               */
/*-------------------------------*/

/*------------------------------------------------------*/
/* constructor/destructor                               */
/*------------------------------------------------------*/

/**
 * RXgN^
 */
BasicSystem::BasicSystem()
{
  int  i, j;

  /* --- clear */
  m_PlayerInput = 0;
  m_PlayerInputTriger = 0;
  m_IsMusicPlaying = false;
  m_PlayingMusic = 0;

  /* --- handle */
  for(i=0; i<BS_SOUND_MAX; i++) {
    m_Sound[i] = INVALID_SOUND;
  }
  for(i=0; i<BS_MUSIC_MAX; i++) {
    m_Music[i] = INVALID_SOUND;
  }
  m_TextureMyShip =
    m_TextureEnemy =
    m_TextureBonus =
    m_TextureBullet =
    m_TextureBack =
    m_TexturePanel =
    m_TextureTitle = INVALID_TEXTURE;
  m_SpriteMyShip =
    m_SpriteMyShot =
    m_SpriteEnemy =
    m_SpriteBonus =
    m_SpriteBullet =
    m_SpriteBack =
    m_SpritePanel =
    m_SpriteTitle = INVALID_SPRITE;

  /* --- fobObZ[W̕ێGAm */
  for(i=0; i<DEBUGMESSAGELIST; i++) {
    m_DebugMessage[i] = new unsigned char[128];
    for(j=0; j<128; j++) {
      m_DebugMessage[i][j] = 0;
    }
  }
}


/**
 * fXgN^(n)
 */
BasicSystem::~BasicSystem()
{
  int i;

  StopMusic();

  /* --- fobObZ[W̉ */
  for(i=0; i<DEBUGMESSAGELIST; i++) {
    delete m_DebugMessage[i];
  }

  /* --- Luna nh̉(cĂ) */
  Release();
}



/*------------------------------------------------------*/
/* interface                                            */
/*------------------------------------------------------*/

/* ----------------------------------------- */
/* --- Init() */
/**
 * Sprite, Input, Sound Ƃ Luna NX̏
 * eNX`ǂݍ݁ATEhǂݍ݂ƂƂsB
 * Ƃ construct ɂ͍sꂸ Init ɂčsB
 *
 * @return 
 *
 */
void BasicSystem::Init()
{
  int  i;

  /* --- TEhnh̏ */
  for(i=0; i<BS_SOUND_MAX; i++) {
    m_Sound[i] = LunaSound::CreateFromFile(sound_name[i]);
  }
  for(i=0; i<BS_MUSIC_MAX; i++) {
    m_Music[i] = LunaSound::CreateFromFileForStream(music_name[i]);
  }

  /* --- eNX`nh̏ */
  m_TextureMyShip = LunaTexture::CreateFromFile(texture_name[0],
						FORMAT_TEXTURE,
						COLORKEY_GREEN);
  m_TextureEnemy = LunaTexture::CreateFromFile(texture_name[1],
					       FORMAT_TEXTURE,
					       COLORKEY_GREEN);
  m_TextureBonus = LunaTexture::CreateFromFile(texture_name[2],
					       FORMAT_TEXTURE,
					       COLORKEY_GREEN);
  m_TextureBullet = LunaTexture::CreateFromFile(texture_name[3],
						FORMAT_TEXTURE,
						COLORKEY_GREEN);
  m_TextureBack = LunaTexture::CreateFromFile(texture_name[4],
					      FORMAT_TEXTURE,
					      COLORKEY_GREEN);
  m_TexturePanel = LunaTexture::CreateFromFile(texture_name[5],
					       FORMAT_TEXTURE,
					       D3DCOLOR_XRGB(0x33,0x33,0x33));
  m_TextureTitle = LunaTexture::CreateFromFile(texture_name[6],
					       FORMAT_TEXTURE,
					       COLORKEY_GREEN);

  /* --- XvCgnh̏ */
  m_SpriteMyShip = LunaSprite::Create(BS_SPBUF_MYSHIP * 8,
				      PRIM_VERTEX_UV1,
				      false);
  m_SpriteMyShot = LunaSprite::Create(BS_SPBUF_MYSHOT * 8,
				      PRIM_VERTEX_UV1,
				      false);
  m_SpriteEnemy = LunaSprite::Create(BS_SPBUF_ENEMY * 8,
				     PRIM_VERTEX_UV1,
				     false);
  m_SpriteBonus = LunaSprite::Create(BS_SPBUF_BONUSSHIP * 8,
				     PRIM_VERTEX_UV1,
				     false);
  m_SpriteBullet = LunaSprite::Create(BS_SPBUF_BULLET * 8,
				      PRIM_VERTEX_UV1,
				      false);
  m_SpriteBack = LunaSprite::Create(BS_SPBUF_BACKGROUND * 8,
				    PRIM_VERTEX_UV1,
				    false);
  m_SpritePanel = LunaSprite::Create(BS_SPBUF_PANEL * 8,
				     PRIM_VERTEX_UV1,
				     false);
  m_SpriteTitle = LunaSprite::Create(BS_SPBUF_TITLE * 8,
				     PRIM_VERTEX_UV1,
				     false);

  /* --- XvCgɃeNX`t */
  if (m_TextureMyShip != INVALID_TEXTURE) {
    LunaSprite::AttatchTexture(m_SpriteMyShip, 0, m_TextureMyShip);
  }
  if (m_TextureBullet != INVALID_TEXTURE) {
    LunaSprite::AttatchTexture(m_SpriteMyShot, 0, m_TextureBullet);
    LunaSprite::AttatchTexture(m_SpriteBullet, 0, m_TextureBullet);
  }
  if (m_TextureEnemy != INVALID_TEXTURE) {
    LunaSprite::AttatchTexture(m_SpriteEnemy, 0, m_TextureEnemy);
  }
  if (m_TextureBonus != INVALID_TEXTURE) {
    LunaSprite::AttatchTexture(m_SpriteBonus, 0, m_TextureBonus);
  }
  if (m_TextureBack != INVALID_TEXTURE) {
    LunaSprite::AttatchTexture(m_SpriteBack, 0, m_TextureBack);
  }
  if (m_TexturePanel != INVALID_TEXTURE) {
    LunaSprite::AttatchTexture(m_SpritePanel, 0, m_TexturePanel);
  }
  if (m_TextureTitle != INVALID_TEXTURE) {
    LunaSprite::AttatchTexture(m_SpriteTitle, 0, m_TextureTitle);
  }

  /* --- J[L[nm */
  Luna3D::SetColorkeyEnable( true );
  /* --- At@uh^Cv */
  Luna3D::SetBlendingType( BLEND_NORMAL );

}

/* ----------------------------------------- */
/* --- Release() */
/**
 * Luna Ǘnh
 *
 * @return 
 *
 */
void BasicSystem::Release()
{
  int  i;

  /* --- Sound */
  for(i=0; i<BS_SOUND_MAX; i++) {
    if (m_Sound[i] != INVALID_SOUND) {
      LunaSound::Release(m_Sound[i]);
      m_Sound[i] = INVALID_SOUND;
    }
  }
  for(i=0; i<BS_MUSIC_MAX; i++) {
    if (m_Music[i] != INVALID_SOUND) {
      LunaSound::Release(m_Music[i]);
      m_Music[i] = INVALID_SOUND;
    }
  }
  /* --- Sprite */
  if (m_SpriteMyShip != INVALID_SPRITE) {
    LunaSprite::Release(m_SpriteMyShip);
    m_SpriteMyShip = INVALID_SPRITE;
  }
  if (m_SpriteMyShot != INVALID_SPRITE) {
    LunaSprite::Release(m_SpriteMyShot);
    m_SpriteMyShot = INVALID_SPRITE;
  }
  if (m_SpriteEnemy != INVALID_SPRITE) {
    LunaSprite::Release(m_SpriteEnemy);
    m_SpriteEnemy = INVALID_SPRITE;
  }
  if (m_SpriteBonus != INVALID_SPRITE) {
    LunaSprite::Release(m_SpriteBonus);
    m_SpriteBonus = INVALID_SPRITE;
  }
  if (m_SpriteBullet != INVALID_SPRITE) {
    LunaSprite::Release(m_SpriteBullet);
    m_SpriteBullet = INVALID_SPRITE;
  }
  if (m_SpriteBack != INVALID_SPRITE) {
    LunaSprite::Release(m_SpriteBack);
    m_SpriteBack = INVALID_SPRITE;
  }
  if (m_SpritePanel != INVALID_SPRITE) {
    LunaSprite::Release(m_SpritePanel);
    m_SpritePanel = INVALID_SPRITE;
  }
  if (m_SpriteTitle != INVALID_SPRITE) {
    LunaSprite::Release(m_SpriteTitle);
    m_SpriteTitle = INVALID_SPRITE;
  }
  /* --- Texture */
  if (m_TextureMyShip != INVALID_TEXTURE) {
    LunaTexture::Release(m_TextureMyShip);
    m_TextureMyShip = INVALID_TEXTURE;
  }
  if (m_TextureEnemy != INVALID_TEXTURE) {
    LunaTexture::Release(m_TextureEnemy);
    m_TextureEnemy = INVALID_TEXTURE;
  }
  if (m_TextureBonus != INVALID_TEXTURE) {
    LunaTexture::Release(m_TextureBonus);
    m_TextureBonus = INVALID_TEXTURE;
  }
  if (m_TextureBullet != INVALID_TEXTURE) {
    LunaTexture::Release(m_TextureBullet);
    m_TextureBullet = INVALID_TEXTURE;
  }
  if (m_TextureBack != INVALID_TEXTURE) {
    LunaTexture::Release(m_TextureBack);
    m_TextureBack = INVALID_TEXTURE;
  }
  if (m_TexturePanel != INVALID_TEXTURE) {
    LunaTexture::Release(m_TexturePanel);
    m_TexturePanel = INVALID_TEXTURE;
  }
  if (m_TextureTitle != INVALID_TEXTURE) {
    LunaTexture::Release(m_TextureTitle);
    m_TextureTitle = INVALID_TEXTURE;
  }
}

/* ----------------------------------------- */
/* --- Tick() */
/**
 * t[Ƃ̍ƂsB
 * ̓Iɂ Input ǂݎƃXvCgobt@̈ꊇ`B
 *
 * @return 
 *
 */
void BasicSystem::Tick()
{
  /* --- vC[͏ */
  read_pad();

  /* --- XvCg̕` */
  if (Luna3D::BeginScene()) {
    /* --- fobO`FbNfBXvC */
#if DEBUG
    display_debug_strings();
#endif
    LunaSprite::Rendering(m_SpriteBack);
    LunaSprite::Rendering(m_SpriteMyShip);
    LunaSprite::Rendering(m_SpriteMyShot);
    LunaSprite::Rendering(m_SpriteEnemy);
    LunaSprite::Rendering(m_SpriteBonus);
    LunaSprite::Rendering(m_SpriteBullet);
    LunaSprite::Rendering(m_SpritePanel);
    LunaSprite::Rendering(m_SpriteTitle);
    /* - */
    Luna3D::EndScene();
  }

  /* --- t[ɌXvCg̃Zbg */
  LunaSprite::ResetBuffer(m_SpriteMyShip);
  LunaSprite::ResetBuffer(m_SpriteMyShot);
  LunaSprite::ResetBuffer(m_SpriteEnemy);
  LunaSprite::ResetBuffer(m_SpriteBonus);
  LunaSprite::ResetBuffer(m_SpriteBullet);
  LunaSprite::ResetBuffer(m_SpriteBack);
  LunaSprite::ResetBuffer(m_SpritePanel);
  LunaSprite::ResetBuffer(m_SpriteTitle);

}

/* ----------------------------------------- */
/* --- DisplayUpdate() */
/**
 * XvCgobt@̍XV
 * Sync EFCg̑OɓăR[
 *
 * @return 
 *
 */
void BasicSystem::DisplayUpdate()
{
  /* --- t[ɌXvCg̃Abvf[g */
  LunaSprite::UpdateBuffer(m_SpriteBack);
  LunaSprite::UpdateBuffer(m_SpriteMyShip);
  LunaSprite::UpdateBuffer(m_SpriteMyShot);
  LunaSprite::UpdateBuffer(m_SpriteEnemy);
  LunaSprite::UpdateBuffer(m_SpriteBonus);
  LunaSprite::UpdateBuffer(m_SpriteBullet);
  LunaSprite::UpdateBuffer(m_SpritePanel);
  LunaSprite::UpdateBuffer(m_SpriteTitle);
}

/* ----------------------------------------- */
/* --- GetKeyInput() */
/**
 * ͂nB
 *
 * @return 
 *
 */
unsigned long BasicSystem::GetKeyInput()
{
  return(m_PlayerInput);
}

/* ----------------------------------------- */
/* --- GetKeyInputTriger() */
/**
 * ͂̃gK[󋵂nB
 *
 * @return 
 *
 */
unsigned long BasicSystem::GetKeyInputTriger()
{
  return(m_PlayerInputTriger);
}

/* ----------------------------------------- */
/* --- GetBulletSprite() */
/**
 * GeGtFNgp̃XvCgobt@nB
 *
 * @return LunaSprite |C^
 *
 */
LSPRITE BasicSystem::GetBulletSprite()
{
  return(m_SpriteBullet);
}

/* ----------------------------------------- */
/* --- GetMyShipSprite() */
/**
 * @\p̃XvCgobt@nB
 *
 * @return LunaSprite |C^
 *
 */
LSPRITE BasicSystem::GetMyShipSprite()
{
  return(m_SpriteMyShip);
}

/* ----------------------------------------- */
/* --- GetMyShotSprite() */
/**
 * @e\p̃XvCgobt@nB
 *
 * @return LunaSprite |C^
 *
 */
LSPRITE BasicSystem::GetMyShotSprite()
{
  return(m_SpriteMyShot);
}

/* ----------------------------------------- */
/* --- GetEnemySprite() */
/**
 * GQ̕\p̃XvCgobt@nB
 *
 * @return LunaSprite |C^
 *
 */
LSPRITE BasicSystem::GetEnemySprite()
{
  return(m_SpriteEnemy);
}

/* ----------------------------------------- */
/* --- GetBonusShipSprite() */
/**
 * {[iXD\p̃XvCgobt@nB
 *
 * @return LunaSprite |C^
 *
 */
LSPRITE BasicSystem::GetBonusShipSprite()
{
  return(m_SpriteBonus);
}

/* ----------------------------------------- */
/* --- GetPanelSprite() */
/**
 * XRApl\p̃XvCgobt@nB
 *
 * @return LunaSprite |C^
 *
 */
LSPRITE BasicSystem::GetPanelSprite()
{
  return(m_SpritePanel);
}

/* ----------------------------------------- */
/* --- GetTitleSprite() */
/**
 * ^Cgʕ\p̃XvCgobt@nB
 *
 * @return LunaSprite |C^
 *
 */
LSPRITE BasicSystem::GetTitleSprite()
{
  return(m_SpriteTitle);
}

/* ----------------------------------------- */
/* --- GetBackGroundSprite() */
/**
 * Q[wi\p̃XvCgobt@nB
 *
 * @return LunaSprite |C^
 *
 */
LSPRITE BasicSystem::GetBackGroundSprite()
{
  return(m_SpriteBack);
}

/* ----------------------------------------- */
/* --- PlaySoundEffect() */
/**
 * w肵ԍ̌ʉ炵܂
 *
 * @param num [in] 炷ʉ̔ԍ
 * @return 
 *
 */
void BasicSystem::PlaySoundEffect(int num)
{
  if ((num >= 0) && (num < BS_SOUND_MAX)) {
    /* --- nh̃`FbN */
    if (m_Sound[num] != INVALID_SOUND) {
      LunaSound::Play(m_Sound[num]);
    }
  }
}

/* ----------------------------------------- */
/* --- PlayMusic() */
/**
 * w肵ԍ̃TEhXg[tJn
 *
 * @param num [in] 炷ʉ̔ԍ
 * @return 
 *
 */
void BasicSystem::PlayMusic(int num)
{
  if ((num >= 0) && (num < BS_SOUND_MAX)) {
    /* --- t~ */
    if (m_IsMusicPlaying) {
      StopMusic();
    }
    /* --- nh̃`FbN */
    if (m_Music[num] != INVALID_SOUND) {
      LunaSound::Play(m_Music[num]);
      m_PlayingMusic = num;
      m_IsMusicPlaying = true;
    }
  }
}

/* ----------------------------------------- */
/* --- StopMusic() */
/**
 * ݃Xg[t炻~
 *
 * @return 
 *
 */
void BasicSystem::StopMusic()
{
  if (m_IsMusicPlaying) {
    LunaSound::Stop(m_Music[m_PlayingMusic]);
    m_IsMusicPlaying = false;
    m_PlayingMusic = 0;
  }
}

/* ----------------------------------------- */
/* --- AddDebugMessage() */
/**
 * fobOp\̒ǉ
 *
 * @param line [in] fobO\ string
 * @return 
 *
 */
void BasicSystem::AddDebugMessage(unsigned char *line)
{
  int  i;

  for(i=0; i<DEBUGMESSAGELIST; i++) {
    if (m_DebugMessage[i][0] == 0) {
      strncpy((char*)m_DebugMessage[i], (char*)line, 127);
      m_DebugMessage[i][127] = 0;
      break;
    }
  }  
}


/*------------------------------------------------------*/
/* local work                                           */
/*------------------------------------------------------*/

/* ----------------------------------------- */
/* --- display_debug_strings() */
/**
 * fobOp̕\
 *
 * @param line [in] fobO\ string
 * @return 
 *
 */
void  BasicSystem::display_debug_strings()
{
  int  i, j, y;

  y = 0;
  Luna3D::FontBegin( "lr SVbN", 12 );
  for(i=0; i<DEBUGMESSAGELIST; i++) {
    if (m_DebugMessage[i][0] != 0) {
      Luna3D::FontDraw(0,
		       y,
		       RGB(255,255,255),
		       RGB(0,0,100),
		       (char*)m_DebugMessage[i]);
      y += 12;
      for(j=0; j<128; j++) {
	m_DebugMessage[i][j] = 0;
      }
    }
  }
  Luna3D::FontEnd();
}


/* ----------------------------------------- */
/* --- read_pad() */
/**
 * L[ƃWCXeBbN̓ǂݎ
 * ǂݎ m_PlayerInput,  m_PlayerInputTriger ɕێ
 *
 * @return 
 *
 */
void  BasicSystem::read_pad()
{
  JOYSTICKDATA Joy;
  unsigned long  key_input, joystick_input, input;
  
  /* --- ̓foCXXV */
  LunaInput::Refresh();

  /* --- L[{[h̓ǂݎ */
  key_input = 0;
  if (LunaInput::GetKeyData(DIK_UP)) {
    key_input |= BS_INPUT_UP;
  }
  if (LunaInput::GetKeyData(DIK_DOWN)) {
    key_input |= BS_INPUT_DOWN;
  }
  if (LunaInput::GetKeyData(DIK_LEFT)) {
    key_input |= BS_INPUT_LEFT;
  }
  if (LunaInput::GetKeyData(DIK_RIGHT)) {
    key_input |= BS_INPUT_RIGHT;
  }
  if (LunaInput::GetKeyData(DIK_Z)) {
    key_input |= BS_INPUT_BUTTON1;
  }
  if (LunaInput::GetKeyData(DIK_X)) {
    key_input |= BS_INPUT_BUTTON2;
  }
  if (LunaInput::GetKeyData(DIK_C)) {
    key_input |= BS_INPUT_BUTTON3;
  }
  if (LunaInput::GetKeyData(DIK_V)) {
    key_input |= BS_INPUT_BUTTON4;
  }

  /* --- WCXeBbN(pbh)̓ǂݎ */
  joystick_input = 0;
  if ( LunaInput::GetJoystickData( 0, &Joy ) ) {
    if (Joy.Py < (-65536/3)) {
      joystick_input |= BS_INPUT_UP;
    }
    if (Joy.Py > (+65536/3)) {
      joystick_input |= BS_INPUT_DOWN;
    }
    if (Joy.Px < (-65536/3)) {
      joystick_input |= BS_INPUT_LEFT;
    }
    if (Joy.Px > (+65536/3)) {
      joystick_input |= BS_INPUT_RIGHT;
    }
    if (Joy.Button[0]) {
      joystick_input |= BS_INPUT_BUTTON1;
    }
    if (Joy.Button[1]) {
      joystick_input |= BS_INPUT_BUTTON2;
    }
    if (Joy.Button[2]) {
      joystick_input |= BS_INPUT_BUTTON3;
    }
    if (Joy.Button[3]) {
      joystick_input |= BS_INPUT_BUTTON4;
    }
  }

  /* --- Ⴆ */
  key_input |= joystick_input;

  /* --- gK[o */
  m_PlayerInputTriger = (key_input ^ m_PlayerInput);
  m_PlayerInputTriger &= key_input;
  /* --- ʏ */
  m_PlayerInput = key_input;
  
}


